@@ -2,7 +2,7 @@
  *
  *  AT chat library with GLib integration
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -102,6 +102,7 @@ struct at_chat {
 	gboolean in_read_handler;		/* Re-entrancy guard */
 	gboolean in_notify;
 	GSList *terminator_list;		/* Non-standard terminator */
+	guint16 terminator_blacklist;		/* Blacklisted terinators */
 };
 
 struct _GAtChat {
@@ -151,6 +152,7 @@ static void at_notify_destroy(gpointer user_data)
 	struct at_notify *notify = user_data;
 
 	g_slist_foreach(notify->nodes, at_notify_node_destroy, NULL);
+	g_slist_free(notify->nodes);
 	g_free(notify);
 }
 
@@ -489,6 +491,12 @@ static void at_chat_add_terminator(struct at_chat *chat, char *terminator,
 	chat->terminator_list = g_slist_prepend(chat->terminator_list, info);
 }
 
+static void at_chat_blacklist_terminator(struct at_chat *chat,
+						GAtChatTerminator terminator)
+{
+	chat->terminator_blacklist |= 1 << terminator;
+}
+
 static gboolean check_terminator(struct terminator_info *info, char *line)
 {
 	if (info->len == -1 && !strcmp(line, info->terminator))
@@ -511,7 +519,8 @@ static gboolean at_chat_handle_command_response(struct at_chat *p,
 
 	for (i = 0; i < size; i++) {
 		struct terminator_info *info = &terminator_table[i];
-		if (check_terminator(info, line)) {
+		if (check_terminator(info, line) &&
+				(p->terminator_blacklist & 1 << i) == 0) {
 			at_chat_finish_command(p, info->success, line);
 			return TRUE;
 		}
@@ -1438,6 +1447,15 @@ void g_at_chat_add_terminator(GAtChat *chat, char *terminator,
 	at_chat_add_terminator(chat->parent, terminator, len, success);
 }
 
+void g_at_chat_blacklist_terminator(GAtChat *chat,
+						GAtChatTerminator terminator)
+{
+	if (chat == NULL || chat->group != 0)
+		return;
+
+	at_chat_blacklist_terminator(chat->parent, terminator);
+}
+
 gboolean g_at_chat_set_wakeup_command(GAtChat *chat, const char *cmd,
 					unsigned int timeout, unsigned int msec)
 {
