@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
 
@@ -50,9 +49,7 @@ static const char *none_prefix[] = { NULL };
 
 struct nokia_data {
 	GAtChat *modem;
-	GAtChat *control;
-	struct ofono_gprs *gprs;
-	struct ofono_gprs_context *gc;
+	GAtChat *aux;
 };
 
 static int nokia_probe(struct ofono_modem *modem)
@@ -78,17 +75,17 @@ static void nokia_remove(struct ofono_modem *modem)
 
 	ofono_modem_set_data(modem, NULL);
 
-	g_at_chat_unref(data->modem);
-	g_at_chat_unref(data->control);
+	/* Cleanup after hot-unplug */
+	g_at_chat_unref(data->aux);
 
 	g_free(data);
 }
 
 static void nokia_debug(const char *str, void *user_data)
 {
-        const char *prefix = user_data;
+	const char *prefix = user_data;
 
-        ofono_info("%s%s", prefix, str);
+	ofono_info("%s%s", prefix, str);
 }
 
 static GAtChat *open_device(struct ofono_modem *modem,
@@ -112,6 +109,7 @@ static GAtChat *open_device(struct ofono_modem *modem,
 	syntax = g_at_syntax_new_gsm_permissive();
 	chat = g_at_chat_new(channel, syntax);
 	g_at_syntax_unref(syntax);
+
 	g_io_channel_unref(channel);
 
 	if (chat == NULL)
@@ -123,39 +121,20 @@ static GAtChat *open_device(struct ofono_modem *modem,
 	return chat;
 }
 
-static void nokia_disconnect(gpointer user_data)
+static void cfun_enable(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_modem *modem = user_data;
 	struct nokia_data *data = ofono_modem_get_data(modem);
 
 	DBG("");
 
-	if (data->gc)
-		ofono_gprs_context_remove(data->gc);
+	if (!ok) {
+		g_at_chat_unref(data->modem);
+		data->modem = NULL;
 
-	g_at_chat_unref(data->modem);
-	data->modem = NULL;
-
-	data->modem = open_device(modem, "Modem", "Modem: ");
-	if (data->modem == NULL)
-		return;
-
-	g_at_chat_set_disconnect_function(data->modem,
-						nokia_disconnect, modem);
-
-	ofono_info("Reopened GPRS context channel");
-
-	data->gc = ofono_gprs_context_create(modem, 0, "atmodem", data->modem);
-
-	if (data->gprs && data->gc)
-		ofono_gprs_add_context(data->gprs, data->gc);
-}
-
-static void cfun_enable(gboolean ok, GAtResult *result, gpointer user_data)
-{
-	struct ofono_modem *modem = user_data;
-
-	DBG("");
+		g_at_chat_unref(data->aux);
+		data->aux = NULL;
+	}
 
 	ofono_modem_set_powered(modem, ok);
 }
@@ -170,20 +149,20 @@ static int nokia_enable(struct ofono_modem *modem)
 	if (data->modem == NULL)
 		return -EINVAL;
 
-	g_at_chat_set_disconnect_function(data->modem,
-						nokia_disconnect, modem);
-
-	data->control = open_device(modem, "Control", "Control: ");
-	if (data->control == NULL) {
+	data->aux = open_device(modem, "Aux", "Aux: ");
+	if (data->aux == NULL) {
 		g_at_chat_unref(data->modem);
 		data->modem = NULL;
 		return -EIO;
 	}
 
-	g_at_chat_send(data->control, "ATE0 +CMEE=1", none_prefix,
+	g_at_chat_send(data->modem, "ATE0 &C0 +CMEE=1", NULL,
 						NULL, NULL, NULL);
 
-	g_at_chat_send(data->control, "AT+CFUN=1", none_prefix,
+	g_at_chat_send(data->aux, "ATE0 &C0 +CMEE=1", NULL,
+						NULL, NULL, NULL);
+
+	g_at_chat_send(data->aux, "AT+CFUN=1", none_prefix,
 					cfun_enable, modem, NULL);
 
 	return -EINPROGRESS;
@@ -196,8 +175,8 @@ static void cfun_disable(gboolean ok, GAtResult *result, gpointer user_data)
 
 	DBG("");
 
-	g_at_chat_unref(data->control);
-	data->control = NULL;
+	g_at_chat_unref(data->aux);
+	data->aux = NULL;
 
 	if (ok)
 		ofono_modem_set_powered(modem, FALSE);
@@ -209,19 +188,16 @@ static int nokia_disable(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
-	if (data->modem) {
-		g_at_chat_cancel_all(data->modem);
-		g_at_chat_unregister_all(data->modem);
-		g_at_chat_unref(data->modem);
-		data->modem = NULL;
-	}
+	g_at_chat_cancel_all(data->modem);
+	g_at_chat_unregister_all(data->modem);
 
-	if (data->control == NULL)
-		return 0;
+	g_at_chat_unref(data->modem);
+	data->modem = NULL;
 
-	g_at_chat_cancel_all(data->control);
-	g_at_chat_unregister_all(data->control);
-	g_at_chat_send(data->control, "AT+CFUN=4", none_prefix,
+	g_at_chat_cancel_all(data->aux);
+	g_at_chat_unregister_all(data->aux);
+
+	g_at_chat_send(data->aux, "AT+CFUN=4", none_prefix,
 					cfun_disable, modem, NULL);
 
 	return -EINPROGRESS;
@@ -234,35 +210,45 @@ static void nokia_pre_sim(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
-	ofono_devinfo_create(modem, 0, "atmodem", data->control);
-	sim = ofono_sim_create(modem, 0, "atmodem", data->control);
+	ofono_devinfo_create(modem, 0, "atmodem", data->aux);
+	sim = ofono_sim_create(modem, 0, "atmodem", data->aux);
 
 	if (sim)
 		ofono_sim_inserted_notify(sim, TRUE);
 }
 
 static void nokia_post_sim(struct ofono_modem *modem)
+{
+	struct nokia_data *data = ofono_modem_get_data(modem);
+	struct ofono_gprs *gprs;
+	struct ofono_gprs_context *gc;
+
+	DBG("%p", modem);
+
+	ofono_phonebook_create(modem, 0, "atmodem", data->aux);
+
+	ofono_sms_create(modem, OFONO_VENDOR_OPTION_HSO,
+					"atmodem", data->aux);
+
+	gprs = ofono_gprs_create(modem, OFONO_VENDOR_NOKIA,
+					"atmodem", data->aux);
+	gc = ofono_gprs_context_create(modem, 0, "atmodem", data->modem);
+
+	if (gprs && gc)
+		ofono_gprs_add_context(gprs, gc);
+}
+
+static void nokia_post_online(struct ofono_modem *modem)
 {
 	struct nokia_data *data = ofono_modem_get_data(modem);
 
 	DBG("%p", modem);
 
 	ofono_netreg_create(modem, OFONO_VENDOR_NOKIA,
-					"atmodem", data->control);
+					"atmodem", data->aux);
 
-	ofono_sms_create(modem, OFONO_VENDOR_OPTION_HSO,
-					"atmodem", data->control);
 	ofono_ussd_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
-					"atmodem", data->control);
-	ofono_phonebook_create(modem, 0, "atmodem", data->control);
-
-	data->gprs = ofono_gprs_create(modem, OFONO_VENDOR_NOKIA,
-					"atmodem", data->control);
-
-	data->gc = ofono_gprs_context_create(modem, 0, "atmodem", data->modem);
-
-	if (data->gprs && data->gc)
-		ofono_gprs_add_context(data->gprs, data->gc);
+					"atmodem", data->aux);
 }
 
 static struct ofono_modem_driver nokia_driver = {
@@ -273,6 +259,7 @@ static struct ofono_modem_driver nokia_driver = {
 	.disable	= nokia_disable,
 	.pre_sim	= nokia_pre_sim,
 	.post_sim	= nokia_post_sim,
+	.post_online	= nokia_post_online,
 };
 
 static int nokia_init(void)
