@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -605,13 +605,19 @@ static void mw_cphs_mbdn_changed(int id, void *userdata)
 			mw_cphs_mbdn_read_cb, mw);
 }
 
+const struct ofono_phone_number *__ofono_message_waiting_get_mbdn(
+					struct ofono_message_waiting *mw,
+					unsigned int index)
+{
+	return &mw->mailbox_number[index];
+}
+
 static void mw_mbi_read_cb(int ok, int total_length, int record,
 				const unsigned char *data,
 				int record_length, void *userdata)
 {
 	struct ofono_message_waiting *mw = userdata;
 	int i, err;
-	const unsigned char *st;
 
 	if (!ok || record_length < 4) {
 		ofono_error("Unable to read mailbox identifies "
@@ -645,9 +651,8 @@ out:
 	 * Mailbox numbers located in Byte 1, bits 6 & 5,
 	 * Check for Activated & Allocated
 	 */
-	st = ofono_sim_get_cphs_service_table(mw->sim);
-
-	if (st && bit_field(st[0], 4, 2) == 3) {
+	if (__ofono_sim_cphs_service_available(mw->sim,
+					SIM_CPHS_SERVICE_MAILBOX_NUMBERS)) {
 		ofono_sim_read(mw->sim_context, SIM_EF_CPHS_MBDN_FILEID,
 				OFONO_SIM_FILE_STRUCTURE_FIXED,
 				mw_cphs_mbdn_read_cb, mw);
