@@ -194,7 +194,7 @@ static enum ofono_sim_password_type puk2pin(enum ofono_sim_password_type type)
 	case OFONO_SIM_PASSWORD_SIM_PUK2:
 		return OFONO_SIM_PASSWORD_SIM_PIN2;
 	case OFONO_SIM_PASSWORD_PHNET_PUK:
-		return OFONO_SIM_PASSWORD_PHNET_PUK;
+		return OFONO_SIM_PASSWORD_PHNET_PIN;
 	case OFONO_SIM_PASSWORD_PHNETSUB_PUK:
 		return OFONO_SIM_PASSWORD_PHNETSUB_PIN;
 	case OFONO_SIM_PASSWORD_PHSP_PUK:
@@ -1732,6 +1732,8 @@ static void sim_efphase_read_cb(int ok, int length, int record,
 
 static void sim_initialize_after_pin(struct ofono_sim *sim)
 {
+	sim->context = ofono_sim_context_create(sim);
+
 	ofono_sim_read(sim->context, SIM_EFPHASE_FILEID,
 			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
 			sim_efphase_read_cb, sim);
@@ -1933,12 +1935,8 @@ skip_efpl:
 						&sim->language_prefs);
 
 	/* Proceed with sim initialization if we're not merely updating */
-	if (!sim->language_prefs_update) {
-		if (sim->context == NULL)
-			sim->context = ofono_sim_context_create(sim);
-
+	if (!sim->language_prefs_update)
 		__ofono_sim_recheck_pin(sim);
-	}
 
 	sim->language_prefs_update = FALSE;
 }
@@ -2181,6 +2179,12 @@ ofono_bool_t __ofono_sim_service_available(struct ofono_sim *sim,
 	return FALSE;
 }
 
+ofono_bool_t __ofono_sim_cphs_service_available(struct ofono_sim *sim,
+						int cphs_service)
+{
+	return sim_cphs_is_active(sim->cphs_service_table, cphs_service);
+}
+
 static void sim_inserted_update(struct ofono_sim *sim)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
@@ -2219,6 +2223,13 @@ static void sim_free_early_state(struct ofono_sim *sim)
 
 static void sim_free_main_state(struct ofono_sim *sim)
 {
+	int i;
+
+	for (i = 0; i < OFONO_SIM_PASSWORD_INVALID; i++)
+		sim->pin_retries[i] = -1;
+
+	memset(sim->locked_pins, 0, sizeof(sim->locked_pins));
+
 	if (sim->imsi) {
 		g_free(sim->imsi);
 		sim->imsi = NULL;
@@ -2259,7 +2270,10 @@ static void sim_free_main_state(struct ofono_sim *sim)
 		sim->efsst_length = 0;
 	}
 
+	sim->phase = OFONO_SIM_PHASE_UNKNOWN;
+	sim->cphs_phase = OFONO_SIM_CPHS_PHASE_NONE;
 	sim->mnc_length = 0;
+	memset(sim->cphs_service_table, 0, 2);
 
 	if (sim->efimg) {
 		g_free(sim->efimg);
